#define CORE_NAME "irq"

#include <hermit.h>
#include <herrno.h>
#include <htypes.h>
#include <irq.h>

struct irq_ops *irq_ops;

static struct irq_desc irq_desc[64];

int request_irq(int irq, irq_handler_t handler, void *data)
{
	if (irq_desc[irq].handler) {
		dev_err("irq:%d was already requested!\n", irq);
		return -H_EINVAL;
	}

	irq_desc[irq].handler = handler;
	irq_desc[irq].data = data;

	irq_ops->request_irq(irq);

	return 0;
}

void free_irq(int irq)
{
	irq_ops->free_irq(irq);

	irq_desc[irq].handler = NULL;
	irq_desc[irq].data = NULL;
}

void irq_handler(void)
{
	int irq;

	irq_ops->stop_all();

	irq = irq_ops->get_pendding_irq();
	dev_dbg("IRQ[%d]:\n", irq);
	if (irq_desc[irq].handler)
		irq_desc[irq].handler(irq, irq_desc[irq].data);
	else
		dev_err("unknown irq!!\n");

	irq_ops->start_all();
}

void common_handler(void)
{
	dev_err("no support exp\n");
}

void irq_enable_all(void)
{
	if (irq_ops)
		irq_ops->start_all();
}

void irq_disable_all(void)
{
	if (irq_ops)
		irq_ops->stop_all();
}
